import collections
from typing import Optional, Tuple
from base64 import b64encode

import requests


class BaseAPI:
    def __init__(self, username: str, password: str, fqdn: str, endpoint: str, dmPartitionId: int, timeout: Tuple[int, int]):
        self._username = username
        self._password = password
        self._fqdn = fqdn
        self._endpoint = endpoint
        self._requests_timeout = timeout
        self._dmPartitionId = dmPartitionId

    # build basic auth manually since auth parameter in requests is not accepted by MobileIron Cloud API
    def _build_headers(self) -> dict:
        auth_str = b64encode(bytes(f"{self._username}:{self._password}", "utf-8")).decode("ascii")
        return {
            "Authorization": "Basic {}".format(
                auth_str
            )
        }

    def _get_url(self, call_name: Optional[str] = None) -> str:
        return f"https://{self._fqdn}/api/v1/{self._endpoint}" if call_name is None else f"https://{self._fqdn}/api/v1/{self._endpoint}/{call_name}"

    def _call(self,
              call_name: Optional[str] = None,
              http_method: str = "get",
              params: str = None,
              json_value: object = None,
              header_params: dict = {}) -> requests.Response:

        url = self._get_url(call_name=call_name)
        headers = self._build_headers()
        self.extend(headers, header_params)

        return self._execute_call(url=url,
                                  http_method=http_method,
                                  headers=headers,
                                  params=params,
                                  json_value=json_value)

    def _execute_call(self,
                      url: str,
                      http_method: str = None,
                      headers: dict = None,
                      params: str = None,
                      json_value: object = None):
        response = None
        if http_method == 'get':
            if params is not None:
                url = f"{url}?{params}"
            response = requests.get(url, headers=headers, timeout=self._requests_timeout)
        elif http_method == 'post':
            response = requests.post(url, headers=headers, json=json_value, timeout=self._requests_timeout)
        elif http_method == 'put':
            response = requests.put(url, headers=headers, json=json_value, timeout=self._requests_timeout)
        elif http_method == 'delete':
            response = requests.delete(url, headers=headers, json=json_value, timeout=self._requests_timeout)
        response.raise_for_status()
        return response

    @staticmethod
    def extend(*args):
        if args is not None:
            result = None
            if type(args[0]) is collections.OrderedDict:
                result = collections.OrderedDict()
            else:
                result = {}
            for arg in args:
                result.update(arg)
            return result
        return {}



from mobileiron_api.api.base_api import BaseAPI
from typing import List, Optional, Tuple, Dict

from mobileiron_api.api.helpers.filters import request_search_filter
from mobileiron_api.api.helpers.helpers import convert_times


class UserManagementAPI(BaseAPI):
    def __init__(self, username: str, password: str, fqdn: str, dmPartitionId: int, timeout: Tuple[int, int]):
        super(UserManagementAPI, self).__init__(username, password, fqdn, "account", dmPartitionId, timeout)

    # https://help.ivanti.com/mi/help/en_us/cld/76/api/Content/MobileIronCloudCustomerIntegrationAPIGuide/User%20API%20Calls.htm#_Toc507756843 {Reference Broken, to see scroll down}
    def get_user_profile_from_email(self,
                                   email: str,
                                   ) -> Optional[Dict]:
        filters = {
            'UID': email
        }
        params = request_search_filter(key='fq', params=filters)
        response = self._call(params=params)
        return convert_times(response.json())

    # https://help.ivanti.com/mi/help/en_us/cld/76/api/Content/MobileIronCloudCustomerIntegrationAPIGuide/User%20API%20Calls.htm?Highlight=UID%3D#_Toc507756843
    # Sorting results are not implemented : ->
    # https://help.ivanti.com/mi/help/en_us/cld/76/api/Content/MobileIronCloudCustomerIntegrationAPIGuide/MobileIron%20Cloud%20API%20Basics.htm#_Toc507756799
    def get_user_profiles_bulk(self,
                              rows: int = 100,
                              start: int = 0) -> Optional[Dict]:

        filters = {
            'rows': rows,
            'start': start
        }
        params = request_search_filter(params=filters)
        response = self._call(params=params)
        return convert_times(response.json())



from mobileiron_api.api.device_management_api import DeviceManagementAPI
from mobileiron_api.api.user_management_api import UserManagementAPI
from typing import Optional, Tuple


class MobileIronCloudAPI(object):
    device_management_api: DeviceManagementAPI
    user_management_api: UserManagementAPI

    def __init__(self, username: str, password: str, fqdn: str, dmPartitionId: int, timeout: Tuple[int, int] = (10, 60)):
        self._username = username
        self._password = password
        self._fqdn = fqdn
        self.device_management_api = DeviceManagementAPI(username=username, password=password, fqdn=fqdn, dmPartitionId=dmPartitionId,
                                                         timeout=timeout)
        self.user_management_api = UserManagementAPI(username=username, password=password, fqdn=fqdn, dmPartitionId=dmPartitionId,
                                                     timeout=timeout)



from datetime import datetime, timedelta
import math


def to_datetime(epoch_ms: int):
    try:
        return datetime.fromtimestamp(epoch_ms/1000).strftime("%Y-%m-%dT%H:%M:%SZ")
    except:
        raise ValueError(f"Wrong format (%Y-%m-%dT%H:%M:%SZ) for epoch_ms: {epoch_ms}")


def is_epoch_time(value: int) -> bool:
    if isinstance(value, int):
        if value > 0 and len(str(value)) == 13 and str(value)[:2] == "16":
            return True

    return False


def convert_times(json_data: dict) -> dict:
    for key, value in json_data.items():
        if is_epoch_time(value):
            json_data[key] = to_datetime(value)
        if isinstance(value, dict):
            convert_times(value)
        elif isinstance(value, list):
            for val in value:
                if isinstance(val, str):
                    pass
                elif isinstance(val, list):
                    pass
                else:
                    convert_times(val)

    return json_data



from mobileiron_api.api.base_api import BaseAPI
from datetime import datetime, timedelta

from typing import List, Optional, Tuple, Dict

from mobileiron_api.api.helpers.filters import request_search_filter
from mobileiron_api.api.helpers.helpers import convert_times


class DeviceManagementAPI(BaseAPI):
    def __init__(self, username: str, password: str, fqdn: str, dmPartitionId: int, timeout: Tuple[int, int]):
        super(DeviceManagementAPI, self).__init__(username, password, fqdn, "device", dmPartitionId, timeout)

    # https://help.ivanti.com/mi/help/en_us/cld/76/api/Content/MobileIronCloudCustomerIntegrationAPIGuide/Device%20API%20Calls.htm#_Toc507757067
    def get_device_profile_from_email(self,
                                      email: str,
                                      ) -> Optional[Dict]:
        filters = {
            'emailAddress': email,
            'dmPartitionId': self._dmPartitionId
        }
        params = request_search_filter('fq', filters)
        response = self._call(params=params)
        return convert_times(json_data=response.json())

    # https://help.ivanti.com/mi/help/en_us/cld/76/api/Content/MobileIronCloudCustomerIntegrationAPIGuide/Device%20API%20Calls.htm#_Toc507757067
    def get_device_profile_from_id(self,
                                   id: int
                                   ) -> Optional[Dict]:
        filters = {
            'deviceId': id,
            'dmPartitionId': self._dmPartitionId
        }
        params = request_search_filter('fq', filters)
        response = self._call(params=params)
        return convert_times(json_data=response.json())

    # https://help.ivanti.com/mi/help/en_us/cld/76/api/Content/MobileIronCloudCustomerIntegrationAPIGuide/Device%20API%20Calls.htm#_Toc507757067
    def get_device_profile_from_mac(self,
                                    wifimac: str
                                    ) -> Optional[Dict]:
        filters = {
            'wifiMacAddress': wifimac,
            'dmPartitionId': self._dmPartitionId
        }
        params = request_search_filter('fq', filters)
        response = self._call(params=params)
        return convert_times(json_data=response.json())

    # https://help.ivanti.com/mi/help/en_us/cld/76/api/Content/MobileIronCloudCustomerIntegrationAPIGuide/Device%20API%20Calls.htm#_Toc507757067
    def get_device_profiles_bulk(self,
                                rows: int = 100,
                                start: int = 0) -> Optional[Dict]:
        """
        Returns the results of an XQL Query.
        :param rows: Integer to determine how many results will be returned in a call.
        :param start: Offset which the results will be shown off of.
        :return:  Dictionary of results
        """
        filters = {
            'rows': rows,
            'start': start,
            'dmPartitionId': self._dmPartitionId
        }
        params = request_search_filter(params=filters)
        response = self._call(params=params)
        return convert_times(response.json())



from enum import Enum
from typing import List, Any, Optional


def request_search_filter(key: Optional[str] = None, params: Optional[dict] = None) -> str:
    """
    Creates a filter.

    :param key: the dictionary key to hold the stringfied version of url parameters
    :param params: parameters that need to be passed in the url (decoded)
    :return: the filter
    """
    # Convert parameters because the API does not support urlencoded parameters
    filter = "&".join(["{0}={1}".format(str(x), str(y)) for x, y in params.items()])

    return f"{key}={filter}" if key is not None else f"{filter}"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mobileiron_api', 'mobileiron_api.api', 'mobileiron_api.api.helpers']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.28.0,<3.0.0']

setup_kwargs = {
    'name': 'mobileiron-api',
    'version': '0.1.6',
    'description': '',
    'long_description': None,
    'author': 'n1ls',
    'author_email': 'lawiet47@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
